@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2018 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2020 Ahmet Öztürk (aoz_2@yahoo.com)
 
     This file is part of Lifeograph.
 
@@ -46,7 +46,6 @@ AppWindow::AppWindow( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>&
 
     Cipher::init();
 
-    UndoManager::m = new UndoManager;
     Diary::d = new Diary;
     ListData::colrec = new ListData::Colrec;
 
@@ -69,7 +68,9 @@ AppWindow::AppWindow( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>&
         builder->get_widget( "L_infobar", m_L_info );
         builder->get_widget( "B_infobar", m_B_info );
 
-        m_view_login = new ViewLogin;
+        builder->get_widget( "B_back", m_B_back );
+
+        UI_login = new UILogin;
     }
     catch( ... )
     {
@@ -80,27 +81,31 @@ AppWindow::AppWindow( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>&
     m_Hb_main->set_decoration_layout( ":minimize,maximize,close" );
 #endif
 
+    // SIGNALS
+    m_IB_info->signal_response().connect(
+            [ this ]( int r ){ handle_infobar_response( r ); } );
+
     m_A_logout_wo_saving = Lifeograph::p->add_action( "logout_wo_saving",
-            sigc::bind( sigc::mem_fun( *this, &AppWindow::logout ), false ) );
+            [ this ](){ logout( false ); } );
 
     m_A_toggle_auto_logout = Lifeograph::p->add_action_bool( "toggle_auto_logout",
-            sigc::mem_fun( *this, &AppWindow::handle_auto_logout_toggled ), false );
+            [ this ](){ handle_auto_logout_toggled(); }, false );
 
     m_A_show_preferences = Lifeograph::p->add_action( "preferences",
-            sigc::ptr_fun( &DialogPreferences::create ) );
+            [ this ](){ DialogPreferences::create(); } );
+
+    Lifeograph::p->add_action( "enable_editing", [ this ](){ UI_diary->enable_editing(); } );
+
+    Lifeograph::p->add_action( "refresh_panels", [ this ](){ refresh_panels(); } );
 
-    m_A_enable_editing = Lifeograph::p->add_action( "enable_editing",
-            [ this ]() { panel_diary->enable_editing(); } );
     Lifeograph::p->set_accel_for_action( "app.enable_editing", "F2" );
-
-    m_IB_info->signal_response().connect(
-            sigc::mem_fun( this, &AppWindow::handle_infobar_response ) );
+    Lifeograph::p->set_accel_for_action( "app.refresh_panels", "F5" );
 
     m_Bx_main->remove( *m_IB_info ); // necessary due to GNOME Bugzilla bug 710888
 
-    m_A_show_preferences->set_enabled( ! Lifeograph::p->m_flag_read_only );
+    m_A_show_preferences->set_enabled( ! Lifeograph::settings.rtflag_read_only );
 
-    m_view_login->handle_start();
+    UI_login->handle_start();
 
     update_title();
 }
@@ -108,10 +113,9 @@ AppWindow::AppWindow( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>&
 AppWindow::~AppWindow()
 {
     remove();
-    delete UndoManager::m;
 
-    if( m_view_login )
-        delete m_view_login;
+    if( UI_login )
+        delete UI_login;
 
     if( Diary::d )
         delete Diary::d;
@@ -120,7 +124,7 @@ AppWindow::~AppWindow()
 bool
 AppWindow::on_event( GdkEvent* )
 {
-    if( !m_auto_logout_status )
+    if( m_auto_logout_freeze_level == 0 )
     {
         // restart
         m_seconds_remaining = Lifeograph::settings.idletime;
@@ -148,12 +152,31 @@ AppWindow::write_backup()
 {
     PRINT_DEBUG( "AppWindow::write_backup()" );
 
-    m_entry_view->sync();
-
     Result result{ Diary::d->write( Diary::d->get_path() + LOCK_SUFFIX ) };
     return( result == SUCCESS );
 }
 
+void
+AppWindow::refresh_panels()
+{
+    if( not( Diary::d->is_in_edit_mode() ) ) return;
+
+    UI_extra->refresh_active_panel();
+    UI_diary->update_entry_list();
+}
+
+void
+AppWindow::show_entry( Entry* entry )
+{
+    Lifeograph::START_INTERNAL_OPERATIONS();
+
+    UI_entry->show( entry );
+    UI_extra->set_entry( entry );
+    UI_diary->show_in_list( entry );
+
+    Lifeograph::FINISH_INTERNAL_OPERATIONS();
+}
+
 void
 AppWindow::show_about()
 {
@@ -175,8 +198,8 @@ AppWindow::show_about()
 }
 
 void
-AppWindow::show_info( Gtk::MessageType type, const Glib::ustring& text,
-                      const Glib::ustring &button_text, InfoResponse response )
+AppWindow::show_info( Gtk::MessageType type, const Ustring& text,
+                      const Ustring& button_text, InfoResponse response )
 {
     m_L_info->set_markup( text );
     m_B_info->set_visible( !button_text.empty() );
@@ -197,7 +220,7 @@ AppWindow::handle_infobar_response( int response )
         switch( m_resp_cur )
         {
             case RESP_UNDO_REMOVE_DIARY:
-                m_view_login->undo_remove_selected_diary();
+                UI_login->undo_remove_selected_diary();
                 break;
             default:
                 break;
@@ -221,36 +244,101 @@ AppWindow::hide_infobar()
 void
 AppWindow::handle_undo()
 {
-    UndoManager::m->undo();
+    UI_entry->undo();
 }
 
 void
 AppWindow::handle_redo()
 {
-    UndoManager::m->redo();
+    UI_entry->redo();
 }
 
 void
 AppWindow::handle_login()
 {
-    // the following must come before handle_login()s
-    m_auto_logout_status = ( Lifeograph::settings.autologout && Diary::d->is_encrypted() ) ? 0 : 1;
+    if( m_paned_main == nullptr ) // first login
+    {
+        try
+        {
+            // PANES
+            Lifeograph::get_builder()->get_widget( "P_main", m_paned_main );
+            Lifeograph::get_builder()->get_widget( "P_second", m_paned_entry );
 
-    Lifeograph::m_actions_read->set_sensitive( true );
-    Lifeograph::m_actions_edit->set_visible( false );
-    Lifeograph::m_actions_edit->set_sensitive( false );
+            // ICONS
+            Glib::RefPtr< Gtk::IconTheme > theme = Gtk::IconTheme::get_default();
+            theme->append_search_path( Lifeograph::get_icon_dir() );
+
+            Lifeograph::icons->entry_16 = theme->load_icon( "entry-16", 16 );
+            Lifeograph::icons->entry_32 = theme->load_icon( "entry-32", 32 );
+            Lifeograph::icons->entry_parent_16 = theme->load_icon( "entry_parent-16", 16 );
+            Lifeograph::icons->entry_parent_32 = theme->load_icon( "entry_parent-32", 32 );
+            Lifeograph::icons->chapter_16 = theme->load_icon( "chapter-16", 16 );
+            Lifeograph::icons->chapter_32 = theme->load_icon( "chapter-32", 32 );
+
+            Lifeograph::icons->tag_16 = theme->load_icon( "tag-16", 16 );
+            Lifeograph::icons->tag_32 = theme->load_icon( "tag-32", 32 );
+
+            Lifeograph::icons->favorite_16 = theme->load_icon( "favorite-16", 16 );
+
+            Lifeograph::icons->filter_16 = theme->load_icon( "filter-16-symbolic", 16 );
+
+            Lifeograph::icons->todo_auto_16 = theme->load_icon( "todo_auto-16", 16 );
+            Lifeograph::icons->todo_auto_32 = theme->load_icon( "todo_auto-32", 32 );
+            Lifeograph::icons->todo_open_16 = theme->load_icon( "todo_open-16", 16 );
+            Lifeograph::icons->todo_open_32 = theme->load_icon( "todo_open-32", 32 );
+            Lifeograph::icons->todo_progressed_16 = theme->load_icon( "todo_progressed-16", 16 );
+            Lifeograph::icons->todo_progressed_32 = theme->load_icon( "todo_progressed-32", 32 );
+            Lifeograph::icons->todo_done_16 = theme->load_icon( "todo_done-16", 16 );
+            Lifeograph::icons->todo_done_32 = theme->load_icon( "todo_done-32", 32 );
+            Lifeograph::icons->todo_canceled_16 = theme->load_icon( "todo_canceled-16", 16 );
+            Lifeograph::icons->todo_canceled_32 = theme->load_icon( "todo_canceled-32", 32 );
+
+            // PANELS & VIEWS
+            UI_diary = new UIDiary;
+            UI_extra = new UIExtra;
+            UI_entry = new UIEntry;
+        }
+        catch( std::exception &ex )
+        {
+            throw LIFEO::Error( ex.what() );
+        }
+
+        // GEOMETRY
+        int width, dummy;
+        get_size( width, dummy );
+        // try to keep the same ratio between the panels before the resizing occurs:
+        if( width != Lifeograph::settings.width )
+        {
+            Lifeograph::settings.position_pane *=  float( width ) / Lifeograph::settings.width;
+        }
+
+        // check the diary panel's size against the minimum:
+        if( Lifeograph::settings.position_pane > ( width - UIDiary::MIN_WIDTH ) )
+        {
+            PRINT_DEBUG( "Diary panel width set to minimum value" );
+            Lifeograph::settings.position_pane = width - UIDiary::MIN_WIDTH;
+        }
+
+        m_paned_main->set_position( Lifeograph::settings.position_pane );
+        m_paned_entry->set_position( Lifeograph::settings.position_pane_tags );
+
+        // ACTIONS
+        Lifeograph::p->add_action( "undo", [ this ](){ handle_undo(); } );
+        Lifeograph::p->set_accel_for_action( "app.undo", "<Ctrl>Z" );
+        Lifeograph::p->add_action( "redo", [ this ](){ handle_redo(); } );
+        Lifeograph::p->set_accel_for_action( "app.redo", "<Ctrl><Shift>Z" );
+    }
 
     m_St_main->set_visible_child( *m_paned_main );
 
     update_toggle_auto_logout_gui( true );
+
+    m_B_back->set_visible( true );
 }
 
 void
 AppWindow::handle_edit_enabled()
 {
-    Lifeograph::m_actions_edit->set_visible( true );
-    Lifeograph::m_actions_edit->set_sensitive( true );
-
     m_connection_backup = Glib::signal_timeout().connect_seconds(
             sigc::mem_fun( this, &AppWindow::write_backup ), BACKUP_INTERVAL );
 
@@ -260,7 +348,9 @@ AppWindow::handle_edit_enabled()
 void
 AppWindow::handle_logout()
 {
-    if( Lifeograph::loginstatus == Lifeograph::LOGGED_TIME_OUT )
+    m_B_back->set_visible( false );
+
+    if( Diary::d->is_logged_time_out() )
         show_info( Gtk::MESSAGE_INFO, _( STRING::ENTER_PASSWORD_TIMEOUT ) );
     else if( m_flag_info_is_visible )
         hide_infobar();
@@ -270,7 +360,7 @@ AppWindow::handle_logout()
 bool
 AppWindow::handle_idle()
 {
-    if( m_auto_logout_status )
+    if( m_auto_logout_freeze_level > 0 )
         return false;
 
     if( m_seconds_remaining > LOGOUT_COUNTDOWN )
@@ -289,8 +379,8 @@ AppWindow::handle_idle()
     {
         update_auto_logout_count_down();
 
-        Lifeograph::loginstatus = Lifeograph::LOGGED_TIME_OUT;
         logout( true );
+        Diary::d->set_timed_out();
     }
 
     return false;
@@ -299,28 +389,28 @@ AppWindow::handle_idle()
 void
 AppWindow::freeze_auto_logout()
 {
-    if( !m_auto_logout_status )
+    if( m_auto_logout_freeze_level == 0 )
     {
         m_connection_timeout.disconnect();
         m_seconds_remaining = Lifeograph::settings.idletime;
         update_auto_logout_count_down();
     }
 
-    m_auto_logout_status++;
+    m_auto_logout_freeze_level++;
 }
 
 void
 AppWindow::unfreeze_auto_logout()
 {
-    if( Lifeograph::settings.autologout && Diary::d->is_encrypted() &&
-        Lifeograph::loginstatus == Lifeograph::LOGGED_IN && m_auto_logout_status == 1 )
+    if( Lifeograph::settings.idletime > 0 && Diary::d->is_encrypted() &&
+        m_auto_logout_freeze_level == 1 )
     {
         m_connection_timeout = Glib::signal_timeout().connect_seconds(
                 sigc::mem_fun( this, &AppWindow::handle_idle ),
                 Lifeograph::settings.idletime - LOGOUT_COUNTDOWN );
     }
 
-    m_auto_logout_status--;
+    m_auto_logout_freeze_level--;
 }
 
 void
@@ -364,9 +454,12 @@ AppWindow::update_toggle_auto_logout_gui( bool flag_availabilities_too )
 {
     if( flag_availabilities_too )
     {
+        m_auto_logout_freeze_level =
+                ( Lifeograph::settings.idletime && Diary::d->is_encrypted() ) ? 0 : 1;
+
         m_A_toggle_auto_logout->set_enabled( Diary::d->is_encrypted() &&
-                                             Lifeograph::settings.autologout );
-        m_A_logout_wo_saving->set_enabled( Diary::d->is_editing_enabled() );
+                                             Lifeograph::settings.idletime );
+        m_A_logout_wo_saving->set_enabled( Diary::d->is_in_edit_mode() );
     }
 
     m_A_toggle_auto_logout->change_state( ! is_auto_logout_in_use() );
@@ -374,7 +467,7 @@ AppWindow::update_toggle_auto_logout_gui( bool flag_availabilities_too )
 
 // LOG OUT
 bool
-AppWindow::finish_editing( bool opt_save )
+AppWindow::finish_login_session( bool opt_save )
 {
     // SAVING
     Lifeograph::settings.position_pane = m_paned_main->get_position();
@@ -384,10 +477,9 @@ AppWindow::finish_editing( bool opt_save )
     if( ! Diary::d->get_path().empty() )
         Lifeograph::settings.recentfiles.insert( Diary::d->get_path() );
 
-    if( Diary::d->is_editing_enabled() )
+    if( Diary::d->is_in_edit_mode() )
     {
-        m_entry_view->sync();
-        Diary::d->set_last_elem( panel_main->get_cur_elem() );
+        Diary::d->set_last_entry( UI_entry->get_cur_entry() );
 
         if( opt_save )
         {
@@ -433,24 +525,16 @@ AppWindow::finish_editing( bool opt_save )
 
     m_connection_timeout.disconnect();
 
-    // CLEARING
-    // TODO: m_loginstatus = LOGGING_OUT_IN_PROGRESS;
+    Lifeograph::signal_logout().emit(); // only for DialogEvent
 
-    Lifeograph::m_actions_read->set_sensitive( false );
+    Lifeograph::START_INTERNAL_OPERATIONS();
+    UI_diary->handle_logout();
+    UI_entry->handle_logout();
+    UI_extra->handle_logout();
 
-    Lifeograph::s_signal_logout.emit();	// only for DialogEvent
-
-    Lifeograph::s_internaloperation++;
-    panel_main->handle_logout();
-    panel_diary->handle_logout();
-    m_entry_view->handle_logout();
-    panel_extra->handle_logout();
-
-    if( Lifeograph::loginstatus == Lifeograph::LOGGED_IN )
-        Lifeograph::loginstatus = Lifeograph::LOGGED_OUT;
     Diary::d->remove_lock_if_necessary();
     Diary::d->clear();
-    Lifeograph::s_internaloperation--;
+    Lifeograph::FINISH_INTERNAL_OPERATIONS();
 
     return true;
 }
@@ -458,29 +542,28 @@ AppWindow::finish_editing( bool opt_save )
 void
 AppWindow::logout( bool opt_save )
 {
-    panel_main->hide_popover(); // hide if is visible. necessary when logging out without saving
-
-    Lifeograph::p->m_flag_open_directly = false;   // should be reset to prevent logging in again
-    if( finish_editing( opt_save ) )
+    // should be reset to prevent logging in again:
+    Lifeograph::settings.rtflag_open_directly = false;
+    if( finish_login_session( opt_save ) )
     {
         this->handle_logout();
-        m_view_login->handle_logout();
+        UI_login->handle_logout();
         m_St_main->set_visible_child( "login" );
     }
 
-    m_auto_logout_status = 1;
+    m_auto_logout_freeze_level = 1;
 
     update_title();
 }
 
 bool
-AppWindow::confirm_dismiss_element( const DiaryElement* elem, Gtk::Widget* extra_widget )
+AppWindow::confirm_dismiss_element( const Ustring& name, Gtk::Widget* extra_widget )
 {
     Gtk::MessageDialog message( *this, "", false,
                                 Gtk::MESSAGE_WARNING, Gtk::BUTTONS_NONE, true );
     message.set_message(
             Glib::ustring::compose( _( "Are you sure, you want to dismiss %1?" ),
-                                    Glib::ustring::compose( "\"%1\"", elem->get_name() ) ) );
+                                    Glib::ustring::compose( "\"%1\"", name ) ) );
     message.set_secondary_text( _( "This operation cannot be undone!" ) );
     message.add_button( _( "_Cancel" ), Gtk::RESPONSE_CANCEL );
     message.add_button( _( "_DISMISS!" ), Gtk::RESPONSE_ACCEPT );
@@ -500,11 +583,11 @@ AppWindow::confirm_dismiss_element( const DiaryElement* elem, Gtk::Widget* extra
 void
 AppWindow::update_title()
 {
-    set_title( Lifeograph::loginstatus == Lifeograph::LOGGED_IN ?
-            Diary::d->get_name() : PROGRAM_NAME );
+    set_title( Diary::d->is_open() ?
+               STR::compose( PROGRAM_NAME, " - ", Diary::d->get_name() ) : PROGRAM_NAME );
 
     static Gtk::Label label;
-    if( Lifeograph::loginstatus == Lifeograph::LOGGED_IN ) // just hide the title
+    if( Diary::d->is_open() ) // just hide the title
         m_Hb_main->set_custom_title( label );
     else // remove custom widget to show the title
         m_Hb_main->property_custom_title() = nullptr;
@@ -513,135 +596,19 @@ AppWindow::update_title()
 void
 AppWindow::login()
 {
-    if( m_paned_main == nullptr )
-    {
-        try
-        {
-            // PANES
-            Lifeograph::get_builder()->get_widget( "P_main", m_paned_main );
-            Lifeograph::get_builder()->get_widget( "P_second", m_paned_entry );
+    Lifeograph::START_INTERNAL_OPERATIONS();
 
-            // TAG PANEL
-            Lifeograph::get_builder()->get_widget_derived( "TV_all_tags", panel_extra );
+    this->handle_login();       // must come first
+    UI_login->handle_login();
+    UI_diary->handle_login();
+    UI_extra->handle_login();
+    UI_entry->handle_login();   // must come last
 
-            // ICONS
-            Glib::RefPtr< Gtk::IconTheme > theme = Gtk::IconTheme::get_default();
-            theme->append_search_path( Lifeograph::get_icon_dir() );
+    show_entry( Diary::d->get_startup_entry() );
 
-            Lifeograph::icons->entry_16 = Lifeograph::load_icon( "entry-16.png" );
-            Lifeograph::icons->entry_32 = Lifeograph::load_icon( "entry-32.png" );
-            Lifeograph::icons->favorite_16 = Lifeograph::load_icon( "favorite-16.png" );
+    Lifeograph::FINISH_INTERNAL_OPERATIONS();
 
-            Lifeograph::icons->tag_16 = Lifeograph::load_icon( "tag-16.png" );
-            Lifeograph::icons->tag_32 = Lifeograph::load_icon( "tag-32.png" );
-            Lifeograph::icons->tag_theme_16 = Lifeograph::load_icon( "tag_theme-16.png" );
-            Lifeograph::icons->tag_theme_32 = Lifeograph::load_icon( "tag_theme-32.png" );
-
-            Lifeograph::icons->tag_p_16 = Lifeograph::load_icon( "tag_p-16.png" );
-            Lifeograph::icons->tag_p_32 = Lifeograph::load_icon( "tag_p-32.png" );
-            Lifeograph::icons->tag_p_theme_16 = Lifeograph::load_icon( "tag_p_theme-16.png" );
-            Lifeograph::icons->tag_p_theme_32 = Lifeograph::load_icon( "tag_p_theme-32.png" );
-
-            Lifeograph::icons->untagged_16 = Lifeograph::load_icon( "untagged-16.png" );
-            Lifeograph::icons->untagged_32 = Lifeograph::load_icon( "untagged-32.png" );
-            Lifeograph::icons->tag_category_16 = Lifeograph::load_icon( "tag_category-16.png" );
-            Lifeograph::icons->tag_category_32 = Lifeograph::load_icon( "tag_category-32.png" );
-
-            Lifeograph::icons->chapter_free_16 = Lifeograph::load_icon( "chapter_free-16.png" );
-            Lifeograph::icons->chapter_free_32 = Lifeograph::load_icon( "chapter_free-32.png" );
-
-            Lifeograph::icons->chapter_temp_16 = Lifeograph::load_icon( "chapter_temp-16.png" );
-            Lifeograph::icons->chapter_temp_32 = Lifeograph::load_icon( "chapter_temp-32.png" );
-
-            Lifeograph::icons->chapter_ordr_16 = Lifeograph::load_icon( "chapter_ordr-16.png" );
-            Lifeograph::icons->chapter_ordr_32 = Lifeograph::load_icon( "chapter_ordr-32.png" );
-
-            Gtk::IconInfo ii = theme->lookup_icon( "filter-16-symbolic", 16,
-                                                   Gtk::IconLookupFlags( 0 ) );
-            bool res;
-            Lifeograph::icons->filter_16 = ii.load_symbolic( m_paned_main->get_style_context(),
-                                                             res );
-
-            ii = theme->lookup_icon( "go-next-symbolic", 16, Gtk::IconLookupFlags( 0 ) );
-            Lifeograph::icons->next_16 = ii.load_symbolic( m_paned_main->get_style_context(),
-                                                                         res );
-
-            Lifeograph::icons->bullet_16 = Lifeograph::load_icon( "bullet-16.png" );
-            Lifeograph::icons->todo_auto_32 = Lifeograph::load_icon( "todo_auto-32.png" );
-            Lifeograph::icons->todo_open_16 = Lifeograph::load_icon( "todo_open-16.png" );
-            Lifeograph::icons->todo_open_32 = Lifeograph::load_icon( "todo_open-32.png" );
-            Lifeograph::icons->todo_progressed_16 =
-                    Lifeograph::load_icon( "todo_progressed-16.png" );
-            Lifeograph::icons->todo_progressed_32 =
-                    Lifeograph::load_icon( "todo_progressed-32.png" );
-            Lifeograph::icons->todo_done_16 = Lifeograph::load_icon( "todo_done-16.png" );
-            Lifeograph::icons->todo_done_32 = Lifeograph::load_icon( "todo_done-32.png" );
-            Lifeograph::icons->todo_canceled_16 = Lifeograph::load_icon( "todo_canceled-16.png" );
-            Lifeograph::icons->todo_canceled_32 = Lifeograph::load_icon( "todo_canceled-32.png" );
-
-            // PANELS & VIEWS
-            m_filter_view = new FilterView; // this is needed by PanelDiary
-            panel_main = new PanelMain;
-            panel_diary = new PanelDiary;
-            m_diary_view = new DiaryView;
-            m_ctg_tags_view = new CategoryTagsView;
-            m_chapter_view = new ChapterView;
-            m_entry_view = new EntryView;
-            m_tag_view = new TagView;
-        }
-        catch( std::exception &ex )
-        {
-            throw LIFEO::Error( ex.what() );
-        }
-
-        // GEOMETRY
-        int width, dummy;
-        get_size( width, dummy );
-        // try to keep the same ratio between the panels before the resizing occurs:
-        if( width != Lifeograph::settings.width )
-        {
-            Lifeograph::settings.position_pane *=  float( width ) / Lifeograph::settings.width;
-        }
-
-        // check the diary panel's size against the minimum:
-        if( Lifeograph::settings.position_pane > ( width - PanelDiary::MIN_WIDTH ) )
-        {
-            PRINT_DEBUG( "Diary panel width set to minimum value" );
-            Lifeograph::settings.position_pane = width - PanelDiary::MIN_WIDTH;
-        }
-
-        m_paned_main->set_position( Lifeograph::settings.position_pane );
-        m_paned_entry->set_position( Lifeograph::settings.position_pane_tags );
-
-        // ACTIONS
-        Lifeograph::create_action( true, m_action_undo,"Undo",
-                       _( "Undo" ), "",     // this tooltip will be dynamic
-                       Gtk::AccelKey( GDK_KEY_z, Gdk::CONTROL_MASK ),
-                       sigc::mem_fun( *this, &AppWindow::handle_undo ) );
-
-        Lifeograph::create_action( true, m_action_redo, "Redo",
-                       _( "Redo" ), "",     // this tooltip will be dynamic
-                       Gtk::AccelKey( GDK_KEY_z, Gdk::CONTROL_MASK | Gdk::SHIFT_MASK ),
-                       sigc::mem_fun( *this, &AppWindow::handle_redo ) );
-    }
-
-    // LOGIN HANDLERS
-    Lifeograph::s_internaloperation++;
-
-    this->handle_login();   // must come first
-    panel_main->handle_login(); // must come before m_diary_view->handle_login() for header bar order
-    m_view_login->handle_login();
-    m_tag_view->handle_login();
-    m_filter_view->handle_login();
-    m_entry_view->handle_login();
-    m_diary_view->handle_login();
-    m_chapter_view->handle_login();
-    panel_diary->handle_login();
-    panel_extra->handle_login();
-
-    Lifeograph::s_internaloperation--;
-
-    Lifeograph::loginstatus = Lifeograph::LOGGED_IN;
+    // NOTE: LOGGED_IN flag used to be set here, now it is set much earlier
 
     update_title();
 }
