@@ -1,6 +1,6 @@
 /***********************************************************************************
 
-    Copyright (C) 2007-2019 Ahmet Öztürk (aoz_2@yahoo.com)
+    Copyright (C) 2007-2020 Ahmet Öztürk (aoz_2@yahoo.com)
 
     Parts of this file are loosely based on an example gcrypt program
     on http://punkroy.drque.net/
@@ -30,6 +30,7 @@
 #include <iomanip>
 
 #include "helpers.hpp"
+#include "strings.hpp"
 
 
 namespace HELPERS
@@ -69,7 +70,7 @@ Date::Date( const std::string& str_date )
 Glib::Date::Month
 Date::get_month_glib() const
 {
-    switch( ( m_date & MONTH_FILTER ) >> 15 )
+    switch( ( m_date & FILTER_MONTH ) >> 15 )
     {
         case 1:
             return Glib::Date::JANUARY;
@@ -101,7 +102,7 @@ Date::get_month_glib() const
 }
 
 Result
-Date::parse_string( Date::date_t* date, const Ustring& str_date )
+Date::parse_string( date_t* date, const Ustring& str_date )
 {
     char c_cur;
     unsigned int num[ 4 ] = { 0, 0, 0, 0 };  // fourth int is for trailing spaces
@@ -191,16 +192,17 @@ Date::parse_string( Date::date_t* date, const Ustring& str_date )
 
         if( date )  // pass NULL when just used for checking
             *date = date_tmp.m_date;
-
-    }
-    else if( num[ 1 ] )   // ordinal
-    {
-        if( num[ 0 ] > CHAPTER_MAX || num[ 1 ] > ORDER_MAX )
-            return OUT_OF_RANGE;
-
-        if( date )  // pass NULL when just used for checking
-            *date = make_date( num[ 0 ], num[ 1 ] );
     }
+// there is no failsafe way of determining if a string is temporal or ordinal.
+// so support for ordinals is disabled for now.
+//    else if( num[ 1 ] )   // ordinal
+//    {
+//        if( num[ 0 ] > ORDER_MAX || num[ 1 ] > ORDER_MAX )
+//            return OUT_OF_RANGE;
+//
+//        if( date )  // pass NULL when just used for checking
+//            *date = make_ordinal( true, num[ 0 ], num[ 1 ], num[ 2 ] );
+//    }
     else
         return INVALID;
 
@@ -210,13 +212,15 @@ Date::parse_string( Date::date_t* date, const Ustring& str_date )
 Ustring
 Date::format_string( const date_t d, const std::string& format, const char separator )
 {
-    std::stringstream result;
+    std::string result;
 
-    if( d & ORDINAL_FLAG )
+    if( d & FLAG_ORDINAL )
     {
-        result << ( get_ordinal_order( d ) + 1 );
-        if( get_order( d ) )
-            result << "." << get_order( d );
+        result += std::to_string( get_order_1st( d ) + 1 );
+        if( get_order_2nd( d ) )
+            result += ( "." + std::to_string( get_order_2nd( d ) ) );
+        if( get_order_3rd( d ) )
+            result += ( "." + std::to_string( get_order_3rd( d ) ) );
     }
     else
     {
@@ -234,28 +238,33 @@ Date::format_string( const date_t d, const std::string& format, const char separ
                 }
             };
 
-        for( unsigned int i = 0; i < format.size(); i++ )
+        for( unsigned int i = 0; ; i++ )
         {
-            result << std::setfill( '0' ) << std::setw( 2 ) << get_YMD( format[ i ] );
-            if( i != format.size() - 1 )
-                result << separator;
+            const auto&& ymd{ get_YMD( format[ i ] ) };
+            if( ymd < 10 ) result += '0';
+            result += std::to_string( ymd );
+            if( i == format.size() - 1 ) break;
+            result += separator;
         }
     }
-    return result.str();
+    return result;
 }
 
 Ustring
 Date::format_string_dt( const time_t time )
 {
     struct tm* timeinfo = localtime( &time );
-    std::stringstream result;
+    std::string result;
 
-    result << format_string( make_date_from_ctime( timeinfo ) )
-           << ", "
-           << std::setfill( '0' ) << std::setw( 2 ) << timeinfo->tm_hour << ":"
-           << std::setfill( '0' ) << std::setw( 2 ) << timeinfo->tm_min;
+    result += format_string( make_from_ctime( timeinfo ) );
+    result += ", ";
+    if( timeinfo->tm_hour < 10 ) result += '0';
+    result += std::to_string( timeinfo->tm_hour );
+    result += ':';
+    if( timeinfo->tm_min < 10 ) result += '0';
+    result += std::to_string( timeinfo->tm_min );
 
-    return result.str();
+    return result;
 }
 
 // date only
@@ -263,28 +272,41 @@ Ustring
 Date::format_string_d( const time_t time )
 {
     struct tm* timeinfo = localtime( &time );
-    return format_string( make_date_from_ctime( timeinfo ) );
+    return format_string( make_from_ctime( timeinfo ) );
 }
 
 std::string
 Date::get_year_str() const
 {
-    std::stringstream result;
-    result << get_year();
-    return result.str();
+    return std::to_string( get_year() );
 }
 
 Ustring
 Date::get_month_str() const
 {
-    return Glib::Date( 20, get_month_glib(), 2000 ).format_string( "%b" );
+    using namespace LIFEO;
+    switch( get_month() )
+    {
+        case  1: return STR_DATE/SI::JANUARY;
+        case  2: return STR_DATE/SI::FEBRUARY;
+        case  3: return STR_DATE/SI::MARCH;
+        case  4: return STR_DATE/SI::APRIL;
+        case  5: return STR_DATE/SI::MAY;
+        case  6: return STR_DATE/SI::JUNE;
+        case  7: return STR_DATE/SI::JULY;
+        case  8: return STR_DATE/SI::AUGUST;
+        case  9: return STR_DATE/SI::SEPTEMBER;
+        case 10: return STR_DATE/SI::OCTOBER;
+        case 11: return STR_DATE/SI::NOVEMBER;
+        default: return STR_DATE/SI::DECEMBER;
+    }
 }
 
-int
-Date::get_weekday() const
+unsigned int
+Date::get_weekday( const date_t& date )
 {
     // from wikipedia: http://en.wikipedia.org/wiki/Calculating_the_day_of_the_week
-    const unsigned int year = get_year();
+    const unsigned int year = get_year( date );
     const unsigned int century = ( year - ( year % 100 ) ) / 100;
     int c = 2 * ( 3 - ( century % 4 ) );
     int y = year % 100;
@@ -292,15 +314,36 @@ Date::get_weekday() const
 
     static const int t_m[] = { 0, 3, 3, 6, 1, 4, 6, 2, 5, 0, 3, 5 };
 
-    int m = get_month() - 1;
-    int d = ( c + y + t_m[ m ] + get_day() );
+    int m = get_month( date ) - 1;
+    int d = ( c + y + t_m[ m ] + get_day( date ) );
 
-    if( m < 2 && is_leap_year() )  // leap year!
+    if( m < 2 && is_leap_year( date ) )  // leap year!
         d += 6;
 
     return( d % 7 );
 }
 
+unsigned int
+Date::get_yearday( const date_t& date )
+{
+    int result{ 0 };
+    date_t d_m{ make( get_year( date ), 1, 1, 0 ) };
+
+    for( unsigned int i = 1; i < 12; i++ )
+    {
+        if( i < get_month( date ) )
+            result += get_days_in_month( d_m );
+        else
+            break;
+
+        forward_months( d_m, 1 );
+    }
+
+    result += get_day( date );
+
+    return result;
+}
+
 Ustring
 Date::get_weekday_str() const
 {
@@ -319,21 +362,21 @@ Date::get_weekday_str() const
 }
 
 unsigned int
-Date::get_days_in_month() const
+Date::get_days_in_month( const date_t d )
 {
-    switch( get_month() )
+    switch( get_month( d ) )
     {
         case 4: case 6: case 9: case 11:
             return 30;
         case 2:
-            return is_leap_year() ? 29 : 28;
+            return is_leap_year( d ) ? 29 : 28;
     }
 
     return 31;
 }
 
 time_t
-Date::get_ctime( const Date::date_t d )
+Date::get_ctime( const date_t d )
 {
     time_t t;
     time( &t );
@@ -349,34 +392,78 @@ Date::get_ctime( const Date::date_t d )
 }
 
 void
-Date::forward_months( int months )
+Date::forward_months( date_t& d, unsigned int months )
 {
-    months += get_month();
-    m_date &= YEAR_FILTER;   // isolate year
-    const int mod_months = months % 12;
+    months += get_month( d );
+    d &= FILTER_YEAR;   // isolate year
+    const auto mod_months{ months % 12 };
     if( mod_months == 0 )
     {
-        m_date += make_year( ( months / 12 ) - 1);
-        m_date |= 0x60000;  // make month 12
+        d += make_year( ( months / 12 ) - 1 );
+        d |= 0x60000;  // make month 12
     }
     else
     {
-        m_date += make_year( months / 12 );
-        m_date |= make_month( mod_months );
+        d += make_year( months / 12 );
+        d |= make_month( mod_months );
     }
+
+    d |= make_day( 1 );
 }
 
 void
-Date::forward_day()
+Date::backward_months( date_t& d, unsigned int months )
 {
-    unsigned int day = get_day();
-    if( day >= get_days_in_month() )
+    const int m_diff{ int( months ) - int( get_month( d ) ) };
+    d &= FILTER_YEAR;   // isolate year
+    if( m_diff < 0 )
+        d |= make_month( -m_diff );
+    else
     {
-        set_day( 1 );
-        forward_months( 1 );
+        d -= make_year( ( m_diff / 12 ) + 1 );
+        d |= make_month( 12 - ( m_diff % 12 ) );
+    }
+
+    d |= make_day( get_days_in_month( d ) );
+}
+
+void
+Date::backward_to_week_start( date_t& date )
+{
+    const auto wd{ get_weekday( date ) };
+    if( wd > 0 )
+        backward_days( date, wd );
+}
+
+void
+Date::forward_days( date_t& date, unsigned int n_days )
+{
+    const auto day_new = get_day( date ) + n_days;
+    const auto days_in_month{ get_days_in_month( date ) };
+
+    if( day_new > days_in_month )
+    {
+        forward_months( date, 1 );    // sets day to 1, hence -1 below
+        if( day_new > days_in_month + 1 )
+            forward_days( date, day_new - days_in_month - 1 );
     }
     else
-        set_day( day + 1 );
+        set_day( date, day_new );
+}
+
+void
+Date::backward_days( date_t& date, unsigned int n_days )
+{
+    const int day_new = get_day( date ) - n_days;
+
+    if( day_new <= 0 )
+    {
+        backward_months( date, 1 );    // sets day to the last day of previous month
+        if( day_new < 0 )
+            backward_days( date, -day_new );
+    }
+    else
+        set_day( date, day_new );
 }
 
 unsigned int
@@ -389,13 +476,26 @@ Date::calculate_days_between( const Date& date2 ) const
 }
 
 unsigned int
-Date::calculate_months_between( Date::date_t date2 ) const
+Date::calculate_weeks_between( date_t date1, date_t date2 )
 {
-    return calculate_months_between( m_date, date2 );
+    unsigned int dist{ 0 };
+    date_t date_former{ Date::get_pure( date1 < date2 ? date1 : date2 ) };
+    date_t date_latter{ Date::get_pure( date1 < date2 ? date2 : date1 ) };
+
+    backward_to_week_start( date_former );
+    backward_to_week_start( date_latter );
+
+    while( date_former < date_latter )
+    {
+        forward_days( date_former, 7 );
+        dist++;
+    }
+
+    return dist;
 }
 
 unsigned int
-Date::calculate_months_between( Date::date_t date1, Date::date_t date2 )
+Date::calculate_months_between( date_t date1, date_t date2 )
 {
     int dist{ 12 * int( get_year( date2 ) - get_year( date1 ) ) };
     dist += ( get_month( date2 ) - get_month( date1 ) );
@@ -404,7 +504,7 @@ Date::calculate_months_between( Date::date_t date1, Date::date_t date2 )
 }
 
 int
-Date::calculate_months_between_neg( Date::date_t date1, Date::date_t date2 )
+Date::calculate_months_between_neg( date_t date1, date_t date2 )
 {
     int dist{ 12 * int( get_year( date2 ) - get_year( date1 ) ) };
     dist += ( get_month( date2 ) - get_month( date1 ) );
@@ -440,9 +540,9 @@ Color
 midtone( const Color& c1, const Color& c2, float ratio )
 {
     Color midtone;
-    midtone.set_red_u( ( c1.get_red_u() * ratio ) + ( c2.get_red_u() * ( 1.0 - ratio ) ) );
-    midtone.set_green_u( ( c1.get_green_u() * ratio ) + ( c2.get_green_u() * ( 1.0 - ratio ) ) );
-    midtone.set_blue_u( ( c1.get_blue_u() * ratio ) + ( c2.get_blue_u() * ( 1.0 - ratio ) ) );
+    midtone.set_red_u( ( c1.get_red_u() * ( 1.0 - ratio ) ) + ( c2.get_red_u() * ratio ) );
+    midtone.set_green_u( ( c1.get_green_u() * ( 1.0 - ratio ) ) + ( c2.get_green_u() * ratio ) );
+    midtone.set_blue_u( ( c1.get_blue_u() * ( 1.0 - ratio ) ) + ( c2.get_blue_u() * ratio ) );
     midtone.set_alpha( 1.0 );
     return midtone;
 }
@@ -454,7 +554,7 @@ get_exec_path()
 {
     char lpFilename[ 1024 ];
     GetModuleFileName( NULL, lpFilename, 1024 );
-    return( Glib::path_get_dirname( lpFilename ) );
+    return( Glib::path_get_dirname( convert_locale_to_utf8( lpFilename ) ) );
 }
 #endif
 
@@ -472,7 +572,7 @@ get_file_size( std::ifstream& file )
 }
 
 bool
-copy_file_suffix( const std::string& source_path, const std::string& suffix1, int suffix2 )
+copy_file( const std::string& source_path, const std::string& target_path_root, bool f_unique )
 {
     try
     {
@@ -480,23 +580,51 @@ copy_file_suffix( const std::string& source_path, const std::string& suffix1, in
 
         if( file_src )
         {
-            std::stringstream ss;
-            ss << source_path << suffix1;
-            if( suffix2 >= 0 )
-                ss << suffix2;
+            std::string target_path( target_path_root );
             int index( 0 );
 
-            while( access( PATH( ss.str() ).c_str(), F_OK ) == 0 )
+            while( f_unique && access( PATH( target_path ).c_str(), F_OK ) == 0 )
             {
-                ss.str( std::string() );
-                ss.clear();
-                ss << source_path << suffix1;
-                if( suffix2 >= 0 )
-                    ss << suffix2;
-                ss << " (" << ++index << ")";
+                target_path = target_path_root;
+                target_path += ( " (" + std::to_string( ++index ) + ")" );
             }
 
-            Glib::RefPtr< Gio::File > file_dest = Gio::File::create_for_path( ss.str() );
+            Glib::RefPtr< Gio::File > file_dest = Gio::File::create_for_path( target_path );
+            file_src->copy( file_dest, f_unique ? Gio::FILE_COPY_NONE : Gio::FILE_COPY_OVERWRITE );
+        }
+    }
+    catch( ... )
+    {
+        return false;
+    }
+    return true;
+}
+
+bool
+copy_file_suffix( const std::string& source_path, const std::string& suffix1, int suffix2,
+                  bool flag_unique )
+{
+    try
+    {
+        Glib::RefPtr< Gio::File > file_src = Gio::File::create_for_path( source_path );
+
+        if( file_src )
+        {
+            const std::string target_path_root( source_path + suffix1 );
+            std::string target_path( target_path_root );
+            if( suffix2 >= 0 )
+                target_path += std::to_string( suffix2 );
+            int index( 0 );
+
+            while( flag_unique && access( PATH( target_path ).c_str(), F_OK ) == 0 )
+            {
+                target_path = target_path_root;
+                if( suffix2 >= 0 )
+                    target_path += std::to_string( suffix2 );
+                target_path += ( " (" + std::to_string( ++index ) + ")" );
+            }
+
+            Glib::RefPtr< Gio::File > file_dest = Gio::File::create_for_path( target_path );
             file_src->copy( file_dest );
         }
     }
@@ -526,7 +654,7 @@ is_dir( const std::string& path )
 }
 
 void
-get_all_files_in_path( const std::string& path, ListPaths& list )
+get_all_files_in_path( const std::string& path, SetStrings& list )
 {
     Gio::init();
     try
@@ -556,40 +684,150 @@ get_all_files_in_path( const std::string& path, ListPaths& list )
 }
 
 #ifdef _WIN32
-std::string
-convert_filename_to_win32_locale( const std::string& fn )
+String
+convert_filename_to_win32_locale( const String& fn )
 {
     char* buf = g_win32_locale_filename_from_utf8( fn.c_str() );
     std::string path{ buf };
-    delete buf;
+    g_free( buf );
     return path;
 }
 #endif
 
 // TEXT OPERATIONS =================================================================================
-long
-convert_string( const std::string& str )
+std::string
+STR::format_percentage( double percentage )
 {
-    //TODO: add negative number support
-    long result( 0 );   // result
-    for( unsigned int i = 0;
-         i < str.size() && i < 10 && int ( str[ i ] ) >= '0' && int ( str[ i ] ) <= '9';
-         i++ )
+    std::stringstream str;
+    str << std::fixed << std::setprecision( 1 ) << percentage * 100 << '%';
+    // TODO add locale support
+
+    return str.str();
+}
+
+std::string
+STR::format_number( double number )
+{
+    char result[ 32 ];
+    sprintf( result, "%f", number );
+
+    const auto pos_point{ strcspn( result, ".," ) };
+    const auto size{ strlen( result ) };
+
+    if( pos_point == size )
+        return result;
+
+    std::string str; // decimals separator
+    int         decimal_cnt{ 0 };
+
+    for( auto p = pos_point; p > 0; p-- )
     {
-        result = ( result * 10 ) + int ( str[ i ] ) - '0';
+        if( p != pos_point && pos_point > 4 && ( ( pos_point - p ) % 3 ) == 0 )
+            str.insert( str.begin(), ' ' ); // thousands separator for 10 000 or bigger
+
+        str.insert( str.begin(), result[ p - 1 ] );
     }
-    return( result );
+
+    for( auto p = size - 1; p > pos_point; p-- )
+    {
+        if( decimal_cnt > 0 || result[ p ] != '0' )
+            str.insert( str.size() - decimal_cnt++, 1, result[ p ] );
+    }
+    if( decimal_cnt > 0 )
+        str.insert( str.size() - decimal_cnt, 1, '.' );
+
+    return str;
+}
+
+bool
+STR::ends_with ( const std::string& str, const std::string& end )
+{
+    if( str.length() > end.length() )
+    {
+        return( str.compare( str.length() - end.length(), end.length(), end ) == 0 );
+    }
+    else
+    {
+        return false;
+    }
+}
+
+bool
+STR::get_line( const std::string& source, std::string::size_type& o, std::string& line )
+{
+    if( source.empty() || o >= source.size() )
+        return false;
+
+    std::string::size_type o_end{ source.find( '\n', o ) };
+
+    if( o_end == std::string::npos )
+    {
+        line = source.substr( o );
+        o = source.size();
+    }
+    else
+    {
+        line = source.substr( o, o_end - o );
+        o = o_end + 1;
+    }
+
+    return true;
+}
+
+int
+STR::replace( std::string& txt, const String& s_search, const String& s_replace )
+{
+    int occurrences{ 0 };
+
+    for( auto pos = txt.find( s_search );
+         pos != std::string::npos;
+         pos = txt.find( s_search, pos + s_replace.size() ) )
+    {
+        txt.replace( pos, s_search.size(), s_replace );
+        occurrences++;
+    }
+
+    return occurrences;
+}
+
+String
+STR::replace_spaces( const String& txt )
+{
+    std::string output;
+    char ch;
+
+    for( unsigned int pos = 0 ; pos < txt.size(); pos++ )
+    {
+        ch = txt[ pos ];
+        if( ch == ' ' || ch == '\t' )
+            output += '_';
+        else
+            output += ch;
+    }
+
+    return output;
+}
+
+unsigned long
+STR::get_ul( const std::string& line, int& i )
+{
+    unsigned long result{ 0 };
+
+    for( ; i < int( line.size() ) && int ( line[ i ] ) >= '0' && int ( line[ i ] ) <= '9'; i++ )
+        result = ( result * 10 ) + int ( line[ i ] ) - '0';
+
+    return result;
 }
 
 double
-convert_string_d( const std::string& text )
+STR::get_d( const std::string& text )
 {
     //NOTE: this implementation may be a little bit more forgiving than good for health
-    double value{ 0 };
+    double value{ 0.0 };
     //char lf{ '=' }; // =, \, #, $(unit)
-    int divider{ 0 };
-    bool negative{ false };
-    Wchar c;
+    int    divider{ 0 };
+    bool   negative{ false };
+    Wchar  c;
 
     for( Ustring::size_type i = 0; i < text.size(); i++ )
     {
@@ -621,22 +859,79 @@ convert_string_d( const std::string& text )
     if( negative )
         value *= -1;
 
-    PRINT_DEBUG( "parsed value: ", value );
+    return value;
+}
+
+double
+STR::get_d( const std::string& line, int& i )
+{
+    double value{ 0.0 };
+    int    divider{ 0 };
+    bool   negative{ false };
+    bool   f_continue{ true };
+    Wchar  c;
+
+    for( ; i < int( line.size() ) && f_continue; i++ )
+    {
+        c = line[ i ];
+        switch( c )
+        {
+            case ',':
+            case '.':
+                if( !divider ) // note that if divider
+                    divider = 1;
+                break;
+            case '-':
+                negative = true;
+                break;
+            case '0': case '1': case '2': case '3': case '4':
+            case '5': case '6': case '7': case '8': case '9':
+                value *= 10;
+                value += ( c - '0' );
+                if( divider )
+                    divider *= 10;
+                break;
+            default:
+                i--;
+                f_continue = false; // end loop
+                break;
+        }
+    }
+
+    if( divider > 1 )
+        value /= divider;
+    if( negative )
+        value *= -1;
 
     return value;
 }
 
-bool
-str_ends_with ( const std::string& str, const std::string& end )
+Ustring
+STR::lowercase( const Ustring& str )
 {
-    if( str.length() > end.length() )
+    // fixes the Glib problem with the Turkish capital i (İ)
+    Ustring result;
+    for( Ustring::size_type i = 0; i < str.length(); i++ )
     {
-        return( str.compare( str.length() - end.length(), end.length(), end ) == 0 );
-    }
-    else
-    {
-        return false;
+        if( str.at( i ) == L'İ' )
+            result += 'i';
+        else
+            result += char_lower( str.at( i ) );
     }
+
+    return result;
+}
+
+gunichar
+char_lower( gunichar c )
+{
+    return Glib::Unicode::tolower( c );
+}
+
+bool
+is_char_alpha( gunichar c )
+{
+    return Glib::Unicode::isalpha( c );
 }
 
 std::string
@@ -654,18 +949,6 @@ get_env_lang()
     return s_lang_env;
 }
 
-gunichar
-char_lower( gunichar c )
-{
-    return Glib::Unicode::tolower( c );
-}
-
-bool
-is_char_alpha( gunichar c )
-{
-    return Glib::Unicode::isalpha( c );
-}
-
 #ifdef _WIN32
 
 wchar_t*
@@ -931,8 +1214,15 @@ bool set_widget_css( Gtk::Widget* w, const Ustring& css )
 }
 
 // ENTRYCLEAR ======================================================================================
+EntryClear::EntryClear()
+:   Gtk::Entry(), m_pressed( false )
+{
+    signal_icon_press().connect( sigc::mem_fun( this, &EntryClear::handle_icon_press ) );
+    signal_icon_release().connect( sigc::mem_fun( this, &EntryClear::handle_icon_release ) );
+}
+
 EntryClear::EntryClear( BaseObjectType* cobject, const Glib::RefPtr<Gtk::Builder>& )
-    :   Gtk::Entry( cobject ), m_pressed( false )
+:   Gtk::Entry( cobject ), m_pressed( false )
 {
     signal_icon_press().connect( sigc::mem_fun( this, &EntryClear::handle_icon_press ) );
     signal_icon_release().connect( sigc::mem_fun( this, &EntryClear::handle_icon_release ) );
@@ -1032,7 +1322,7 @@ bool is_treepath_less( const Gtk::TreePath& p, const Gtk::TreePath& pb )
             return false;
     }
 
-    return false; // more or equal
+    return true; // if pb has more members apart from the shared ones, it is more
 }
 
 bool is_treepath_more( const Gtk::TreePath& p, const Gtk::TreePath& pe )
@@ -1052,4 +1342,58 @@ bool is_treepath_more( const Gtk::TreePath& p, const Gtk::TreePath& pe )
     return false; // less or equal
 }
 
+// OTHER GTK+ ======================================================================================
+void
+flush_gtk_event_queue()
+{
+    while( Gtk::Main::events_pending() )
+        Gtk::Main::iteration();
+}
+
+Gdk::Rectangle
+get_rectangle_from_event( const GdkEventButton* event )
+{
+    Gdk::Rectangle rect{ int( event->x ), int( event->y ), 0, 0 };
+    return rect;
+}
+
+bool
+select_image_file( Gtk::Window& parent, std::string& uri )
+{
+    auto* file_dialog{ new Gtk::FileChooserDialog( parent, _( "Select Image File" ) ) };
+
+    auto filter_any = Gtk::FileFilter::create();
+    auto filter_img = Gtk::FileFilter::create();
+
+    filter_any->set_name( _( "All Files" ) );
+    filter_any->add_pattern( "*" );
+    filter_img->set_name( _( "Image Files" ) );
+#ifdef _WIN32
+    filter_img->add_pattern( "*.jpg" );
+    filter_img->add_pattern( "*.png" );
+#else
+    filter_img->add_mime_type( "image/jpeg" );
+    filter_img->add_mime_type( "image/png" );
+#endif
+
+    file_dialog->add_filter( filter_any );
+    file_dialog->add_filter( filter_img );
+
+    file_dialog->add_button( Gtk::Stock::CANCEL, Gtk::RESPONSE_CANCEL );
+    file_dialog->add_button( Gtk::Stock::OPEN, Gtk::RESPONSE_OK );
+
+    if( file_dialog->run() == Gtk::RESPONSE_OK )
+    {
+        uri = file_dialog->get_uri();
+
+        delete file_dialog;
+
+        return true;
+    }
+
+    delete file_dialog;
+
+    return false;
+}
+
 } // end of name space
